/* alias.h - Command aliases
 * Copyright 2000-2001 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: alias.h,v 1.12.2.1.2.1 2002/01/24 23:29:35 entrope Exp $
 */

#ifndef ALIAS_H
#define ALIAS_H

#include "hash.h"
#include "common.h"

typedef int (*alias_command_func_t)(const char *alias, const char *command);
typedef int (*command_search_func_t)(const char *command);
typedef int (*delete_command_func_t)(const char *alias);

/* Alias names have to be unique within a service, but you can have
   multiple aliases pointing at a command. commandNodes just store
   the aliases pointing at them. */

struct aliasNode
{
    char *alias;
    char *command;
};

struct commandNode
{
    char *command;
    dict_t aliases;
};

struct serviceNode
{
    const char *service;

    alias_command_func_t alias_func;
    command_search_func_t search_func;
    delete_command_func_t delete_func;

    dict_t aliases;
    dict_t commands;
};

#define ALIAS_CREATED			1
#define ALIAS_DELETED			2
#define ALIAS_MODIFIED			3
#define ALIAS_SERVICE_NOT_FOUND		4
#define ALIAS_COMMAND_NOT_FOUND		5
#define ALIAS_NOT_FOUND			6
#define ALIAS_FOUND_COMMAND		7
#define ALIAS_SELF			8

int alias_register_service(const char *name, command_search_func_t search_func, alias_command_func_t alias_func, delete_command_func_t delete_func);

int add_alias(const char *service, const char *new_alias, char *command);
int del_alias(const char *service, const char *alias);

char* resolve_alias(const char *service, char *alias);
char* command_aliases(const char *service, char *command);
char* cNode_aliases(struct commandNode *cNode);

void read_aliases(void);

void alias_init(void);

#endif
