/* log.h - Diagnostic and error logging
 * Copyright 2000 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: log.h,v 1.8.2.3.2.2 2002/03/04 03:50:28 sailorfrag Exp $
 */

#ifndef LOG_H
#define LOG_H

#include "hash.h"

enum log_type {
    MAIN_LOG = 1,
    DEBUG_LOG,
    CS_LOG,
    OS_LOG,
    NS_LOG,
    G_LOG,
    PC_LOG,
    HS_LOG,
    /* insert new log types here, and add to log.c:logs */
    NUM_LOGS
};

enum log_severity {
    LOG_OTHER = 1,
    LOG_INFO,
    LOG_WARNING,
    LOG_ERROR,
    LOG_NUM_SEVERITIES
};

#define DEFAULT_SEVERITY_MASK 31

#define ENTRY_COUNT	1024

struct logEntry
{
    time_t		time;

    enum log_type	type;
    enum log_severity	severity;

    char 		*text;
    unsigned int	size;
};

struct logSearch
{
    enum log_type	type;
    enum log_severity	severity;

    char 		*glob;

    unsigned int	limit;
};


int init_logs(void);
void reopen_logs(void);
void log(enum log_type lt, enum log_severity ls, char *format, ...);

typedef void (*entry_search_func)(struct logEntry *match, void *extra);

struct logSearch* log_discrim_create(struct userNode *service, struct userNode *user, unsigned int argc, unsigned char *argv[]);
unsigned int log_entry_search(struct logSearch *discrim, entry_search_func esf, void *data);
int log_modify_logging(const char *logname, const char *mods);
int log_get_mask(const char *logname);

#endif
