/* tools.c - miscellaneous utility functions
 * Copyright 2000-2002 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: tools.c,v 1.73.2.35.2.17 2002/05/04 01:34:35 entrope Exp $
 */

#include "config.h"
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif


#include "common.h"
#include "hash.h"
#include "log.h"
#include "policer.h"
#include "recdb.h"

#define NUMNICKLOG 6
#define NUMNICKBASE (1 << NUMNICKLOG)
#define NUMNICKMASK (NUMNICKBASE - 1)

static const char convert2y[] = {
  'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
  'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
  'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
  'w','x','y','z','0','1','2','3','4','5','6','7','8','9','[',']'
};

static const unsigned int convert2n[] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  52,53,54,55,56,57,58,59,60,61, 0, 0, 0, 0, 0, 0, 
   0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
  15,16,17,18,19,20,21,22,23,24,25,62, 0,63, 0, 0,
   0,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
  41,42,43,44,45,46,47,48,49,50,51, 0, 0, 0, 0, 0,

   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

extern struct policer_params *oper_policer_params, *luser_policer_params;

unsigned long int
base64toint(const unsigned char* s, int count)
{
    unsigned int i = 0;
    while (*s && count) {
        i = (i << NUMNICKLOG) + convert2n[*s++];
        count--;
    }
    return i;
}

const char* inttobase64(char* buf, unsigned int v, unsigned int count)
{
  buf[count] = '\0';
  while (count > 0) {
    buf[--count] = convert2y[(v & NUMNICKMASK)];
    v >>= NUMNICKLOG;
  }
  return buf;
}

static oper_func_t *of_list;
static unsigned int of_size = 0, of_used = 0;

void
reg_oper_func(oper_func_t handler)
{
    if (of_used == of_size) {
	if (of_size) {
	    of_size <<= 1;
	    of_list = realloc(of_list, of_size*sizeof(oper_func_t));
	} else {
	    of_size = 8;
	    of_list = malloc(of_size*sizeof(oper_func_t));
	}
    }
    of_list[of_used++] = handler;
}

static void
call_oper_funcs(struct userNode *user)
{
    unsigned int n;

    for (n=0; n<of_used; n++)
    {
	of_list[n](user);
    }
}

void mod_usermode(struct userNode *user, const char *mode_change) {
    int add = 1;
    if (!user || !mode_change || !*mode_change) return;
    while (1) {
#define do_user_mode(FLAG) do { if (add) user->modes |= FLAG; else user->modes &= ~FLAG; } while (0)
	switch (*mode_change++) {
	case 0: return;
	case '+': add = 1; break;
	case '-': add = 0; break;
	case 'o':
	    do_user_mode(FLAGS_OPER);
	    if (add) {
		userList_append(&curr_opers, user);
		if (user->command_policer) {
		    policer_set_params(user->command_policer, oper_policer_params);
		}
		call_oper_funcs(user);
	    } else {
		userList_remove(&curr_opers, user);
		if (user->command_policer) {
		    policer_set_params(user->command_policer, luser_policer_params);
		}
	    }
	    break;
	case 'O': do_user_mode(FLAGS_LOCOP); break;
	case 'i': do_user_mode(FLAGS_INVISIBLE);
	    if (add) invis_clients++; else invis_clients--;
	    break;
	case 'w': do_user_mode(FLAGS_WALLOP); break;
	case 's': do_user_mode(FLAGS_SERVNOTICE); break;
	case 'd': do_user_mode(FLAGS_DEAF); break;
	case 'k': do_user_mode(FLAGS_SERVICE); break;
	case 'g': do_user_mode(FLAGS_GLOBAL); break;
	case 'h': do_user_mode(FLAGS_HELPER); break;
	}
#undef do_user_mode
    }
}

void mod_chanmode(struct chanNode *channel, const char *mode_change, const char *key, int limit)
{
    int add = 1;
    const char *word = mode_change;

    if (!mode_change || !*mode_change) return;
    while (*word != ' ' && *word) word++;
    while (*word == ' ') word++;
    while (1) {
	switch (*mode_change++) {
	case 0:
	case ' ':
	    return;
	case '+': add = 1; break;
	case '-': add = 0; break;
#define do_chan_mode(FLAG) do { if (add) channel->modes |= FLAG; else channel->modes &= ~FLAG; } while (0)
	case 'n': do_chan_mode(MODE_NOPRIVMSGS); break;
	case 't': do_chan_mode(MODE_TOPICLIMIT); break;
	case 's': do_chan_mode(MODE_SECRET); break;
	case 'p': do_chan_mode(MODE_PRIVATE); break;
	case 'i': do_chan_mode(MODE_INVITEONLY); break;
	case 'm': do_chan_mode(MODE_MODERATED); break;
	case 'D': do_chan_mode(MODE_DELAYJOINS); break;
#undef do_chan_mode
	case 'k':
	    if (add) {
		channel->modes |= MODE_KEY;
		if(key)
		{
		    safestrncpy(channel->key, key, sizeof(channel->key));
		}
		else
		{
                    unsigned int n;
		    for (n=0; (*word != ' ' && *word) && (n < sizeof(channel->key)-1); n++) {
			channel->key[n] = *word++;
		    }
		    channel->key[n] = 0;
		}
	    } else {
		/* Gobble up the key to preserve the alignment between
		   letters and arguments. */
		if(!key)
		{
		    while(*word && *word != ' ') word++;
		}

		channel->modes &= ~MODE_KEY;
	    }
	    break;
	case 'l':
	    if (add) {
		channel->modes |= MODE_LIMIT;
		if(limit) {
		    channel->limit = limit;
		} else {
		    channel->limit = strtoul(word, NULL, 0);
		}
		while (*word != ' ' && *word) word++;
		while (*word == ' ') word++;

		if(!channel->limit) {
		    channel->modes &= ~MODE_LIMIT;
		}
	    } else {
		channel->modes &= ~MODE_LIMIT;
	    }
	}
    }
}

int verify_chanmode(const char *modes)
{
    const char *word = modes;
    int add = -1, res, limit;

    res = 1;

    if(!modes) return 0;
    if(!*modes) return 1;

    while(*word != ' ' && *word) word++;
    while(*word == ' ') word++;
    while(1) {
	switch(*modes++) {
	case 0:
	case ' ':
	    /* Reject the mode if there's crap on the end. */
	    if(*word) return 0;
	    return (add == -1) ? 0 : 1;
	case '+':
	    add = 1;
	    break;
	case '-':
	    add = 0;
	    break;
	case 'n':
	case 't':
	case 's':
	case 'p':
	case 'i':
	case 'm':
        case 'D':
	    break;

	case 'k':
            /* Unlike ircu, allow removing a key without specifying it. */
            if(!add) break;
            if(!*word) return 0;
            while(*word && *word != ' ') word++;
            while(*word == ' ') word++;
            break;
	case 'l':
	    /* As in ircu, the limit is only gobbled when setting it. */
	    if(!add) break;
            if(!*word) return 0;
            limit = strtoul(word, NULL, 0);
            while(*word && *word != ' ') word++;
            while(*word == ' ') word++;
            if(limit < 1) return 0;
	    break;
	default:
	    return 0;
	}
    }

    return 0;
}

int
split_line(unsigned char *line, int irc_colon, int argv_size, unsigned char *argv[])
{
    int argc = 0;
    int n;
    while (*line && (argc < argv_size)) {
	while (*line == ' ') *line++ = 0;
	if (*line == ':' && irc_colon && argc > 0) {
	    /* the rest is a single parameter */
	    argv[argc++] = line + 1;
	    break;
	}
        if (!*line) break;
	argv[argc++] = line;
	if (argc >= argv_size) break;
	while (*line != ' ' && *line) line++;
    }
#ifdef NDEBUG
    n = 0;
#else
    for (n=argc; n<argv_size; n++) {
        argv[n] = (unsigned char*)0xFEEDBEEF;
    }
#endif
    return argc;
}

/* This is ircu's mmatch() function, from match.c. */
int mmatch(const char *old_mask, const char *new_mask)
{
  register const char *m = old_mask;
  register const char *n = new_mask;
  const char *ma = m;
  const char *na = n;
  int wild = 0;
  int mq = 0, nq = 0;

  while (1)
  {
    if (*m == '*')
    {
      while (*m == '*')
	m++;
      wild = 1;
      ma = m;
      na = n;
    }

    if (!*m)
    {
      if (!*n)
	return 0;
      for (m--; (m > old_mask) && (*m == '?'); m--)
	;
      if ((*m == '*') && (m > old_mask) && (m[-1] != '\\'))
	return 0;
      if (!wild)
	return 1;
      m = ma;

      /* Added to `mmatch' : Because '\?' and '\*' now is one character: */
      if ((*na == '\\') && ((na[1] == '*') || (na[1] == '?')))
	++na;

      n = ++na;
    }
    else if (!*n)
    {
      while (*m == '*')
	m++;
      return (*m != 0);
    }
    if ((*m == '\\') && ((m[1] == '*') || (m[1] == '?')))
    {
      m++;
      mq = 1;
    }
    else
      mq = 0;

    /* Added to `mmatch' : Because '\?' and '\*' now is one character: */
    if ((*n == '\\') && ((n[1] == '*') || (n[1] == '?')))
    {
      n++;
      nq = 1;
    }
    else
      nq = 0;

/*
 * This `if' has been changed compared to match() to do the following:
 * Match when:
 *   old (m)         new (n)         boolean expression
 *    *               any             (*m == '*' && !mq) ||
 *    ?               any except '*'  (*m == '?' && !mq && (*n != '*' || nq)) ||
 * any except * or ?  same as m       (!((*m == '*' || *m == '?') && !mq) &&
 *                                      toLower(*m) == toLower(*n) &&
 *                                        !((mq && !nq) || (!mq && nq)))
 *
 * Here `any' also includes \* and \? !
 *
 * After reworking the boolean expressions, we get:
 * (Optimized to use boolean shortcircuits, with most frequently occuring
 *  cases upfront (which took 2 hours!)).
 */
    if ((*m == '*' && !mq) ||
	((!mq || nq) && tolower(*m) == tolower(*n)) ||
	(*m == '?' && !mq && (*n != '*' || nq)))
    {
      if (*m)
	m++;
      if (*n)
	n++;
    }
    else
    {
      if (!wild)
	return 1;
      m = ma;

      /* Added to `mmatch' : Because '\?' and '\*' now is one character: */
      if ((*na == '\\') && ((na[1] == '*') || (na[1] == '?')))
	++na;

      n = ++na;
    }
  }
}

int
match_ircglob(const char *text, const char *glob)
{
    unsigned int star_p, q_cnt;
    while (1) {
	switch (*glob) {
	case 0:
	    return !*text;
        case '\\':
            glob++;
            /* intentionally not tolower(...) so people can force
             * capitalization, or we can overload \ in the future */
            if (*text++ != *glob++) return 0;
            break;
	case '*':
        case '?':
            star_p = q_cnt = 0;
            do {
                if (*glob == '*') star_p = 1;
                else if (*glob == '?') q_cnt++;
                else break;
                glob++;
            } while (1);
            while (q_cnt) { if (!*text++) return 0; q_cnt--; }
            if (star_p) {
                /* if this is the last glob character, it will match any text */
                if (!*glob) return 1;
                /* Thanks to the loop above, we know that the next
                 * character is a normal character.  So just look for
                 * the right character.
                 */
                for (; *text; text++) {
                    if ((tolower(*text) == tolower(*glob))
                        && match_ircglob(text+1, glob+1)) {
                        return 1;
                    }
                }
                return 0;
            }
            /* if !star_p, fall through to normal character case,
             * first checking to see if ?s carried us to the end */
            if (!*glob && !*text) return 1;
	default:
	    if (!*text) return 0;
	    while (*text && *glob && *glob != '*' && *glob != '?' && *glob != '\\') {
		if (tolower(*text++) != tolower(*glob++)) return 0;
	    }
	}
    }
}

int
user_matches_glob(struct userNode *user, const char *orig_glob, int include_nick)
{
    char *glob, *marker;
    /* Make a writable copy of the glob */
    glob = alloca(strlen(orig_glob)+1);
    strcpy(glob, orig_glob);
    /* Check the nick, if it's present */
    if (include_nick) {
        if (!(marker = strchr(glob, '!'))) {
            log(MAIN_LOG, LOG_ERROR, "match_glob_user(\"%s\", \"%s\", %d) called, and glob doesn't include a '!'\n", user->nick, orig_glob, include_nick);
            return 0;
        }
        *marker = 0;
        if (!match_ircglob(user->nick, glob)) return 0;
        glob = marker + 1;
    }
    /* Check the ident */
    if (!(marker = strchr(glob, '@'))) {
        log(MAIN_LOG, LOG_ERROR, "match_glob_user(\"%s\", \"%s\", %d) called, and glob doesn't include an '@'\n", user->nick, orig_glob, include_nick);
        return 0;
    }
    *marker = 0;
    if (!match_ircglob(user->ident, glob)) return 0;
    glob = marker + 1;
    /* Now check the host part */
    if (isdigit(*glob) && !glob[strspn(glob, "0123456789./*?")]) {
        /* Looks like an IP-based mask */
        unsigned char userip[20];
        sprintf(userip, "%ld.%ld.%ld.%ld", (user->ip >> 24) & 255, (user->ip >> 16) & 255, (user->ip >> 8) & 255, user->ip & 255);
        return match_ircglob(userip, glob);
    } else {
        /* The host part of the mask isn't IP-based */
        return match_ircglob(user->hostname, glob);
    }
}

/* TODO: glob comparisons - matching a glob against a glob requires
   different code. for now, match_ircglobs is an alias for ircglob.
*/

int
is_ircmask(const char *text)
{
    while (*text && (isalnum((unsigned char)*text) || strchr("-_[]|\\`^{}?*", *text))) text++;
    if (*text++ != '!') return 0;
    while (*text && *text != '@' && !isspace((unsigned char)*text)) text++;
    if (*text++ != '@') return 0;
    while (*text && (isalnum((unsigned char)*text) || strchr(".-?*", *text))) text++;
    return !*text;
}

int
is_gline(const char *text)
{
    if (*text == '@') return 0;
    text += strcspn(text, "@!% \t\r\n");
    if (*text++ != '@') return 0;
    if (!*text) return 0;
    while (*text && (isalnum((unsigned char)*text) || strchr(".-?*", *text))) text++;
    return !*text;
}

int
split_ircmask(char *text, char **nick, char **ident, char **host)
{
    char *start;

    start = text;
    while (isalnum((unsigned char)*text) || strchr("=[]\\`^{}?*", *text)) text++;
    if (*text != '!' || ((text - start) > NICKLEN)) return 0;
    *text = 0;
    if (nick) *nick = start;

    start = ++text;
    while (*text && *text != '@' && !isspace((unsigned char)*text)) text++;
    if (*text != '@' || ((text - start) > USERLEN)) return 0;
    *text = 0;
    if (ident) *ident = start;
    
    start = ++text;
    while (*text && (isalnum((unsigned char)*text) || strchr(".-?*", *text))) text++;
    if (host) *host = start;
    return !*text && ((text - start) <= HOSTLEN) && nick && ident && host;
}

char *
sanitize_ircmask(char *input)
{
    unsigned int length, flag;
    char *mask, *start, *output;

    /* Sanitize everything in place; input *must* be a valid
       hostmask. */
    output = input;
    flag = 0;

    /* The nick is truncated at the end. */
    length = 0;
    mask = input;
    while(*input++ != '!')
    {
	length++;
    }
    if(length > NICKLEN)
    {
	mask += NICKLEN;
	*mask++ = '!';

	/* This flag is used to indicate following parts should
	   be shifted. */
	flag = 1;
    }
    else
    {
	mask = input;
    }

    /* The ident and host must be truncated at the beginning and
       replaced with a '*' to be compatible with ircu. */
    length = 0;
    start = input;
    while(*input++ != '@')
    {
	length++;
    }
    if(length > USERLEN || flag)
    {
	if(length > USERLEN)
	{
	    start = input - USERLEN;
	    *mask++ = '*';
	}
	while(*start != '@')
	{
	    *mask++ = *start++;
	}
	*mask++ = '@';

	flag = 1;
    }
    else
    {
	mask = input;
    }

    length = 0;
    start = input;
    while(*input++)
    {
	length++;
    }
    if(length > HOSTLEN || flag)
    {
	if(length > HOSTLEN)
	{
	    start = input - HOSTLEN;
	    *mask++ = '*';
	}
	while(*start)
	{
	    *mask++ = *start++;
	}
	*mask = '\0';
    }

    return output;
}

char *
generate_hostmask(struct userNode *user, int options)
{
    char *nickname, *ident, *hostname;
    char *mask;
    int len, ii;

    /* figure out string parts */
    if (options & GENMASK_OMITNICK) {
        nickname = NULL;
    } else if (options & GENMASK_USENICK) {
        nickname = user->nick;
    } else {
        nickname = "*";
    }
    if (options & GENMASK_STRICT_IDENT) {
        ident = user->ident;
    } else if (options & GENMASK_ANY_IDENT) {
        ident = "*";
    } else {
        ident = alloca(strlen(user->ident)+2);
        ident[0] = '*';
        strcpy(ident+1, user->ident + ((*user->ident == '~')?1:0));
    }
    hostname = user->hostname;
    if (options & GENMASK_STRICT_HOST) {
        if (options & GENMASK_BYIP) {
            hostname = alloca(32);
            sprintf(hostname, "%lu.%lu.%lu.%lu", (user->ip>>24)&0xFF, (user->ip>>16)&0xFF, (user->ip>>8)&0xFF, user->ip&0xFF);
        } else {
            /* leave hostname as is */
        }
    } else if ((options & GENMASK_BYIP) || !hostname[strspn(hostname, "0123456789.")]) {
        /* Should generate an IP-based hostmask.  By popular acclaim, a /16
         * hostmask is used by default. */
        unsigned masked_ip, mask, masklen;
        masklen = 16;
        mask = ~0 << masklen;
        masked_ip = user->ip & mask;
        hostname = alloca(32);
        if (options & GENMASK_SRVXMASK) {
            sprintf(hostname, "%d.%d.%d.%d/%d", (masked_ip>>24)&0xFF, (masked_ip>>16)&0xFF, (masked_ip>>8)&0xFF, masked_ip&0xFF, masklen);
        } else {
            int ofs = 0;
            for (ii=0; ii<4; ii++) {
                if (masklen) {
                    ofs += sprintf(hostname+ofs, "%d.", (masked_ip>>24)&0xFF);
                    masklen -= 8;
                    masked_ip <<= 8;
                } else {
                    ofs += sprintf(hostname+ofs, "*.");
                }
            }
            /* Truncate the last . */
            hostname[ofs-1] = 0;
        }
    } else {
        int cnt;
	/* This heuristic could be made smarter.  Is it worth the effort? */
	for (ii=cnt=0; hostname[ii]; ii++) {
	    if (hostname[ii] == '.') cnt++;
        }
	if (cnt == 1) {
	    /* only a two-level domain name; leave hostname */
	} else if (cnt == 2) {
	    for (ii=0; user->hostname[ii] != '.'; ii++) ;
	    /* Add 3 to account for the *. and \0. */
            hostname = alloca(strlen(user->hostname+ii)+3);
            sprintf(hostname, "*.%s", user->hostname+ii+1);
	} else {
	    for (cnt=3, ii--; cnt; ii--) {
		if (user->hostname[ii] == '.') cnt--;
	    }
	    /* The loop above will overshoot the dot one character;
	       we skip forward two (the one character and the dot)
	       when printing, so we only add one for the \0. */
	    hostname = alloca(strlen(user->hostname+ii)+1);
	    sprintf(hostname, "*.%s", user->hostname+ii+2);
	}
    }
    /* Emit hostmask */
    len = strlen(ident) + strlen(hostname) + 2;
    if (nickname) {
        len += strlen(nickname) + 1;
        mask = malloc(len);
        sprintf(mask, "%s!%s@%s", nickname, ident, hostname);
    } else {
        mask = malloc(len);
        sprintf(mask, "%s@%s", ident, hostname);
    }
    return mask;
}

static long
TypeLength(char type)
{
    switch (type) {
    case 'y': return 365*24*60*60;
    case 'M': return 31*24*60*60;
    case 'w': return 7*24*60*60;
    case 'd': return 24*60*60;
    case 'h': return 60*60;
    case 'm': return 60;
    case 's': return 1;
    default: return 0;
    }
}

unsigned long
ParseInterval(const unsigned char *interval)
{
    unsigned long seconds = 0;
    int partial = 0;
    char c;

    /* process the string, resetting the count if we find a unit character */
    while ((c = *interval++)) {
	if (isdigit((int)c)) {
	    partial = partial*10 + c - '0';
	} else {
	    seconds += TypeLength(c) * partial;
	    partial = 0;
	}
    }
    /* assume the last chunk is seconds (the normal case) */
    return seconds + partial;
}

static long
GetSizeMultiplier(char type)
{
    switch (type) {
    case 'G': case 'g': return 1024*1024*1024;
    case 'M': case 'm': return 1024*1024;
    case 'K': case 'k': return 1024;
    case 'B': case 'b': return 1;
    default: return 0;
    }
}

unsigned long
ParseVolume(const unsigned char *volume)
{
    unsigned long accum = 0, partial = 0;
    char c;
    while ((c = *volume++)) {
        if (isdigit((int)c)) {
            partial = partial*10 + c - '0';
        } else {
            accum += GetSizeMultiplier(c) * partial;
            partial = 0;
        }
    }
    return accum + partial;
}

int
parse_ipmask(const unsigned char *str, unsigned long *addr, unsigned long *mask)
{
    int accum, pos;
    unsigned long t_a, t_m;

    t_a = t_m = pos = 0;
    if (addr) *addr = t_a;
    if (mask) *mask = t_m;
    while (*str) {
	if (!isdigit(*str)) return 0;
	accum = 0;
	do {
	    accum = (accum * 10) + *str++ - '0';
	} while (isdigit(*str));
	if (accum > 255) return 0;
	t_a = (t_a << 8) | accum;
	t_m = (t_m << 8) | 255;
	pos += 8;
	if (*str == '.') {
	    str++;
            while (*str == '*') {
                str++;
                if (*str == '.') {
		    t_a <<= 8;
		    t_m <<= 8;
		    pos += 8;
		    str++;
		} else if (*str == 0) {
		    t_a <<= 32 - pos;
		    t_m <<= 32 - pos;
		    pos = 32;
		    goto out;
		} else {
		    return 0;
		}
	    }
	} else if (*str == '/') {
	    int start = pos;
	    accum = 0;
	    do {
		accum = (accum * 10) + *str++ - '0';
	    } while (isdigit(*str));
	    while (pos < start+accum && pos < 32) {
		t_a = (t_a << 1) | 0;
		t_m = (t_m << 1) | 1;
		pos++;
	    }
	    if (pos != start+accum) return 0;
	} else if (*str == 0) {
	    break;
	} else {
	    return 0;
	}
    }
  out:
    if (pos != 32) return 0;
    *addr = t_a;
    *mask = t_m;
    return 1;
}

void
unsplit_string(unsigned char *set[], unsigned int max)
{
    unsigned char *str = set[0];
    if (!max) return;
    while (1) {
	while (*str++) ;
	str--;
	if (!--max) break;
	while (!*str) *str++ = ' ';
    }
    *str = 0;
}

void
intervalString(char *output, time_t interval)
{
    int type = 0, count = 0, words = 0, pos = 0;
    char *unit[] =
    {
	"year",
	"month",
	"week",
	"day",
	"hour",
	"minute",
	"second"
    };
    long length[] =
    {
	365 * 24 * 60 * 60,
	31 * 24 * 60 * 60,
	7 * 24 * 60 * 60,
	24 * 60 * 60,
	60 * 60,
	60,
	1
    };

    if(!interval)
    {
	strcpy(output, "0 seconds");
	return;
    }

    while(words < 2 && interval && type < 7)
    {
	if(interval >= length[type])
	{
	    count = interval / length[type];
	    interval = interval % length[type];

	    if(words == 1)
	    {
		memcpy(output + pos, ", and ", 6);
		pos += 6;
	    }
	    pos += sprintf(output + pos, "%d %s", count, unit[type]);
	    if(count > 1)
	    {
		output[pos++] = 's';
	    }

	    words++;
	}

	type++;
    }

    output[pos] = 0;
}

int
getipbyname(const char *name, unsigned long *ip)
{
    struct hostent *he = gethostbyname(name);
    if (he) {
	if (he->h_addrtype != AF_INET) return 0;
	memcpy(ip, he->h_addr_list[0], sizeof(*ip));
	return 1;
    } else {
	return 0;
    }
}

const char * preposition(const char *word)
{
    switch(tolower(word[0]))
    {
    case 'a':
    case 'e':
    case 'i':
    case 'o':
    case 'u':
	return "an";
    default:
	return "a";
    }
}

int
set_policer_param(const char *param, void *data, void *extra)
{
    struct record_data *rd = data;
    const char *str = GET_RECORD_QSTRING(rd);
    if (str) policer_params_set(extra, param, str);
    return 0;
}

DEFINE_LIST(string_buffer, char)

void
string_buffer_append_string(struct string_buffer *buf, const char *tail)
{
    unsigned int len = strlen(tail);
    while (buf->used + len >= buf->size) {
        buf->size <<= 1;
        buf->list = realloc(buf->list, buf->size*sizeof(buf->list[0]));
    }
    memcpy(buf->list + buf->used, tail, len+1);
    buf->used += len;
}

void
string_buffer_replace(struct string_buffer *buf, unsigned int from, unsigned int len, const char *repl)
{
    unsigned int repl_len = strlen(repl);
    if (from > buf->used) return;
    if (len + from > buf->used) len = buf->used - from;
    buf->used = buf->used + repl_len - len;
    if (buf->size <= buf->used) {
        while (buf->used >= buf->size) {
            buf->size <<= 1;
        }
        buf->list = realloc(buf->list, buf->size*sizeof(buf->list[0]));
    }
    memmove(buf->list+from+repl_len, buf->list+from+len, strlen(buf->list+from+len));
    strcpy(buf->list+from, repl);
}

void
tools_cleanup(void)
{
    if (of_list) free(of_list);
}
